<?php
/**
 * Base Plugin Class
 *
 * @package   edd-conditional-success-redirects
 * @copyright Copyright (c) 2021, Easy Digital Downloads
 * @license   GPL2+
 */
class EDD_Conditional_Success_Redirects {

	/**
	 * @var EDD_Conditional_Success_Redirects
	 */
	private static $instance;

	/**
	 * @var string Current version of the plugin.
	 */
	public $version;

	/**
	 * @var string Path to the main plugin file.
	 */
	public $file;

	/**
	 * @var string Plugin basename.
	 */
	public $basename;

	/**
	 * @var string Path to the plugin's directory.
	 */
	public $plugin_dir;

	/**
	 * @var string URL to the plugin's directory.
	 */
	public $plugin_url;

	/**
	 * @var string Path to the plugin's `includes/` directory.
	 */
	public $includes_dir;

	/**
	 * @var string URL to the plugin's `includes/` directory.
	 */
	public $includes_url;

	/**
	 * Main Instance
	 *
	 * Ensures that only one instance exists in memory at any one
	 * time. Also prevents needing to define globals all over the place.
	 *
	 * @since 1.0
	 *
	 */
	public static function instance() {
		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof EDD_Conditional_Success_Redirects ) ) {
			self::$instance = new EDD_Conditional_Success_Redirects;
			self::$instance->setup_globals();
			self::$instance->includes();
			self::$instance->setup_actions();
			self::$instance->licensing();
			self::$instance->load_textdomain();
		}

		return self::$instance;
	}

	/**
	 * Globals
	 *
	 * @since 1.0
	 *
	 * @return void
	 */
	private function setup_globals() {

		$this->version = EDD_CONDITIONAL_SUCCESS_REDIRECTS_VERSION;

		// paths
		$this->file         = EDD_CONDITIONAL_SUCCESS_REDIRECTS_FILE;
		$this->basename     = apply_filters( 'edd_csr_plugin_basenname', plugin_basename( $this->file ) );
		$this->plugin_dir   = apply_filters( 'edd_csr_plugin_dir_path',  plugin_dir_path( $this->file ) );
		$this->plugin_url   = apply_filters( 'edd_csr_plugin_dir_url',   plugin_dir_url ( $this->file ) );

		// includes
		$this->includes_dir = apply_filters( 'edd_csr_includes_dir', trailingslashit( $this->plugin_dir . 'includes'  ) );
		$this->includes_url = apply_filters( 'edd_csr_includes_url', trailingslashit( $this->plugin_url . 'includes'  ) );

	}

	/**
	 * Setup the default hooks and actions
	 *
	 * @since 1.0
	 *
	 * @return void
	 */
	private function setup_actions() {
		// Add sub-menu page
		add_action( 'admin_menu', array( $this, 'add_redirect_options'), 10 );

		do_action( 'edd_csr_setup_actions' );
	}

	/**
	 * Licensing
	 *
	 * @since 1.0
	 */
	private function licensing() {
		if ( class_exists( '\\EDD\\Extensions\\ExtensionRegistry' ) ) {
			add_action( 'edd_extension_license_init', function( \EDD\Extensions\ExtensionRegistry $registry ) {
				$registry->addExtension( $this->file, 'Conditional Success Redirects', 84640, $this->version );
			} );
		} elseif ( class_exists( 'EDD_License' ) ) {
			$license = new EDD_License( __FILE__, 'Conditional Success Redirects', $this->version, 'Easy Digital Downloads', null, null, 84640 );
		}
	}

	/**
	 * Loads the plugin language files
	 *
	 * @access public
	 * @since 1.1
	 * @return void
	 */
	public function load_textdomain() {

		// Set filter for plugin's languages directory
		$lang_dir = dirname( plugin_basename( __FILE__ ) ) . '/languages/';
		$lang_dir = apply_filters( 'edd_csr_languages_directory', $lang_dir );

		// Traditional WordPress plugin locale filter
		$locale        = apply_filters( 'plugin_locale',  get_locale(), 'edd-csr' );
		$mofile        = sprintf( '%1$s-%2$s.mo', 'edd-csr', $locale );

		// Setup paths to current locale file
		$mofile_local  = $lang_dir . $mofile;
		$mofile_global = WP_LANG_DIR . '/edd-conditional-success-redirects/' . $mofile;

		if ( file_exists( $mofile_global ) ) {
			// Look in global /wp-content/languages/edd-conditional-success-redirects/ folder
			load_textdomain( 'edd-csr', $mofile_global );
		} elseif ( file_exists( $mofile_local ) ) {
			// Look in local /wp-content/plugins/edd-conditional-success-redirects/languages/ folder
			load_textdomain( 'edd-csr', $mofile_local );
		} else {
			// Load the default language files
			load_plugin_textdomain( 'edd-csr', false, $lang_dir );
		}
	}

	/**
	 * Include required files.
	 *
	 * @since 1.0
	 *
	 * @return void
	 */
	private function includes() {

		require( $this->includes_dir . 'class-process-redirects.php' );
		require( $this->includes_dir . 'redirect-functions.php' );

		do_action( 'edd_csr_include_files' );

		if ( ! is_admin() ) {
			return;
		}

		require( $this->includes_dir . 'redirect-actions.php' );
		require( $this->includes_dir . 'admin-notices.php' );
		require( $this->includes_dir . 'post-types.php' );

		do_action( 'edd_csr_include_admin_files' );
	}


	/**
	 * Add submenu page
	 *
	 * @since 1.0
	 */
	public function add_redirect_options() {
		add_submenu_page( 'edit.php?post_type=download', __( 'Conditional Success Redirects', 'edd-csr' ), __( 'Conditional Success Redirects', 'edd-csr' ), 'manage_shop_settings', 'edd-redirects', array( $this, 'redirects_page') );
	}


	/**
	 * Redirects page
	 *
	 * @since 1.0
	 */
	public function redirects_page() {

		if ( isset( $_GET['edd-action'] ) && $_GET['edd-action'] == 'edit_redirect' ) {
			require_once $this->includes_dir . 'edit-redirect.php';
		}
		elseif ( isset( $_GET['edd-action'] ) && $_GET['edd-action'] == 'add_redirect' ) {
			require_once $this->includes_dir . 'add-redirect.php';
		}
		else {
			require_once $this->includes_dir . 'class-redirects-table.php';
			$redirects_table = new EDD_CSR_Table();
			$redirects_table->prepare_items();
			?>
			<div class="wrap">
				<h2><?php _e( 'Conditional Success Redirects', 'edd-csr' ); ?><a href="<?php echo esc_url( add_query_arg( array( 'edd-action' => 'add_redirect' ) ) ); ?>" class="add-new-h2"><?php _e( 'Add New', 'edd-csr' ); ?></a></h2>
				<?php do_action( 'edd_csr_redirects_page_top' ); ?>
				<form id="edd-redirects-filter" method="get" action="<?php echo admin_url( 'edit.php?post_type=download&page=edd-redirects' ); ?>">
					<?php $redirects_table->search_box( __( 'Search', 'edd-csr' ), 'edd-redirects' ); ?>

					<input type="hidden" name="post_type" value="download" />
					<input type="hidden" name="page" value="edd-redirects" />

					<?php $redirects_table->views() ?>
					<?php $redirects_table->display() ?>
				</form>
				<?php do_action( 'edd_csr_redirects_page_bottom' ); ?>
			</div>
			<?php
		}
	}

}
